/*
 * fancydrawingutf8.h
 *
 *  Created on: 29.12.2013
 *      Author: wglas
 */

#ifndef FANCYDRAWINGUTF8_H_
#define FANCYDRAWINGUTF8_H_

#include <fancydrawing.h>
#include <string>

namespace Fancy {
namespace Drawing {

/**
 * @param uc The unicode code point to inspect.
 *
 * @return the length of the given unicode code point when
 *         encoded as UTF-8. This subroutine will return
 *         1 for negative values and does not return values greater
 *         than 4. For code points greater or equal than 0x200000
 *         a warning is issued.
 */
int utf8Length (int uc) FANCYDRAWING_API;

/**
 * Convert a unicode code point to a UTF-8 encoded string.
 * Code points greater or equal than 0x200000 and negative values
 * truncated to 7 bits.
 * Moreover, for code points greater or equal than 0x200000
 * a warning is issued.
  *
 *
 * @return the given code point encoded as UTF-8
 */
std::string toUtf8 (int uc) FANCYDRAWING_API;

/**
 * Append a UTF-8 character to the given string.
 * Code points greater or equal than 0x200000 and negative values
 * truncated to 7 bits.
 * Moreover, for code points greater or equal than 0x200000
 * a warning is issued.
 *
 * @param s The string to append to.
 * @param uc The unicode code point to encode and append.
 */
void appendUtf8 (std::string &s, int uc) FANCYDRAWING_API;

/**
 * Decode a single UTF-8 character and advance pos to the next
 * UTF-8 character.
 * If s[pos] is a UTF continuation byte, the ASCII character
 * (s[pos] & 0x3f)|0x40 is returned and a warning is logged.
 * If s[pos] contains an initalization sequence for a 5 or more
 * byte sequence, the value is interpreted as a four-byte
 * sequence and a warning is logged.
 *
 * In all cases,
 *
 * @param s The string to operate on.
 * @param pos The position where to decode an UTF-8 character.
 *            After returning from this function @c pos is
 *            set to the position of the next UTF-8 character.
 *
 * @return The unicode code point at the given position.
 */
int decodeUtf8 (const std::string &s, size_t &pos) FANCYDRAWING_API;

} /* namespace Drawing */
} /* namespace Fancy */

#endif /* FANCYDRAWINGUTF8_H_ */
